@@ -10,18 +10,67 @@ import multiprocessing
 import os
 import psutil
 from concurrent.futures import ThreadPoolExecutor
-from typing import Dict, Any, Optional, List, Union
+from typing import Dict, Any, Optional, List, Union, Tuple
 from collections import deque
 import cv2
 from pathlib import Path
 
+
+# =========================
+# CPU AFFINITY PINNING
+# =========================
+
+def pin_to_cores(worker_id: int, total_workers: int) -> Optional[List[int]]:
+    """Pin worker process to specific CPU cores for cache locality.
+
+    This optimization from cv2_bench.py improves throughput by 15-20%
+    by reducing CPU cache misses when processing frames.
+
+    Args:
+        worker_id: Worker identifier (0-indexed)
+        total_workers: Total number of worker processes
+
+    Returns:
+        List of CPU core indices this worker is pinned to, or None if pinning failed
+    """
+    try:
+        p = psutil.Process()
+        cpu_count = psutil.cpu_count(logical=True)
+        cores_per_worker = max(1, cpu_count // total_workers)
+
+        start_core = worker_id * cores_per_worker
+        end_core = min(start_core + cores_per_worker, cpu_count)
+
+        core_list = list(range(start_core, end_core))
+        if core_list:
+            p.cpu_affinity(core_list)
+            return core_list
+    except Exception:
+        pass
+    return None
+
 from matrice_common.optimize import FrameOptimizer
+from matrice_common.stream.shm_ring_buffer import ShmRingBuffer
 
 from .video_capture_manager import VideoCaptureManager
 from .frame_processor import FrameProcessor
 from .message_builder import StreamMessageBuilder
 from .stream_statistics import StreamStatistics
 
+import os
+os.environ["OMP_NUM_THREADS"] = "1"
+os.environ["KMP_NUM_THREADS"] = "1"
+os.environ["KMP_BLOCKTIME"] = "0"
+os.environ["TBB_NUM_THREADS"] = "1"
+os.environ["OPENBLAS_NUM_THREADS"] = "1"
+os.environ["MKL_NUM_THREADS"] = "1"
+os.environ["VECLIB_MAXIMUM_THREADS"] = "1"
+os.environ["NUMEXPR_NUM_THREADS"] = "1"
+import cv2
+cv2.setNumThreads(1)
+cv2.setUseOptimized(True)
+cv2.ocl.setUseOpenCL(False)
+
 
 class AsyncCameraWorker:
     """Async worker process that handles multiple cameras concurrently.
@@ -41,6 +90,19 @@ class AsyncCameraWorker:
         response_queue: Optional[multiprocessing.Queue] = None,
         frame_optimizer_enabled: bool = True,
         frame_optimizer_config: Optional[Dict[str, Any]] = None,
+        # ================================================================
+        # SHM_MODE: New parameters for shared memory architecture
+        # ================================================================
+        use_shm: bool = True,  # Feature flag (default: existing JPEG behavior)
+        shm_slot_count: int = 1000,  # Ring buffer size per camera (increased for consumer lag)
+        shm_frame_format: str = "BGR",  # "BGR", "RGB", or "NV12"
+        # ================================================================
+        # PERFORMANCE: New parameters for optimized frame capture
+        # ================================================================
+        drop_stale_frames: bool = True,  # Use grab()/grab()/retrieve() pattern for latest frame
+        pin_cpu_affinity: bool = True,   # Pin worker to specific CPU cores
+        total_workers: int = 1,          # Total worker count for CPU affinity calculation
+        buffer_size: int = 1,            # Minimal buffer for low latency (cv2_bench uses 1)
     ):
         """Initialize async camera worker.
 
@@ -52,6 +114,15 @@ class AsyncCameraWorker:
             health_queue: Queue for reporting health status
             command_queue: Queue for receiving dynamic camera commands (add/remove/update)
             response_queue: Queue for sending command responses back to manager
+            frame_optimizer_enabled: Whether to enable frame optimization
+            frame_optimizer_config: Frame optimizer configuration
+            use_shm: Enable SHM mode (raw frames in shared memory, metadata in Redis)
+            shm_slot_count: Number of frame slots per camera ring buffer
+            shm_frame_format: Frame format for SHM storage ("BGR", "RGB", or "NV12")
+            drop_stale_frames: Use grab()/grab()/retrieve() pattern to get latest frame
+            pin_cpu_affinity: Pin worker process to specific CPU cores for cache locality
+            total_workers: Total number of workers (for CPU affinity calculation)
+            buffer_size: VideoCapture buffer size (1 = minimal latency)
         """
         self.worker_id = worker_id
         self.camera_configs = camera_configs
@@ -91,6 +162,69 @@ class AsyncCameraWorker:
         )
         self._last_sent_frame_ids: Dict[str, str] = {}  # stream_key -> last sent frame_id
 
+        # ================================================================
+        # SHM_MODE: Shared memory ring buffer configuration
+        # ================================================================
+        self.use_shm = use_shm
+        self.shm_slot_count = shm_slot_count
+        self.shm_frame_format = shm_frame_format
+
+        # SHM buffers (created on demand per camera)
+        self._shm_buffers: Dict[str, ShmRingBuffer] = {}
+
+        # Track last written frame_idx per camera for FrameOptimizer references
+        self._last_shm_frame_idx: Dict[str, int] = {}
+
+        # Register atexit and signal handlers for SHM cleanup on crash/exit
+        if use_shm:
+            import atexit
+            import signal
+            import sys
+
+            # atexit handler for normal exits
+            atexit.register(self._cleanup_shm_on_exit)
+
+            # Signal handlers for SIGTERM/SIGINT (graceful shutdown)
+            # This ensures SHM is cleaned up even when killed externally
+            def _signal_handler(signum, frame):
+                """Handle SIGTERM/SIGINT for graceful SHM cleanup."""
+                sig_name = signal.Signals(signum).name if hasattr(signal.Signals, 'name') else str(signum)
+                self.logger.info(f"Worker {worker_id}: Received {sig_name}, cleaning up SHM...")
+                self._cleanup_shm_on_exit()
+                # Re-raise the signal to allow normal termination
+                signal.signal(signum, signal.SIG_DFL)
+                os.kill(os.getpid(), signum)
+
+            # Register signal handlers (SIGINT=Ctrl+C, SIGTERM=kill command)
+            signal.signal(signal.SIGINT, _signal_handler)
+            # SIGTERM may not be available on Windows
+            if sys.platform != 'win32':
+                signal.signal(signal.SIGTERM, _signal_handler)
+
+            self.logger.info(f"Worker {worker_id}: SHM mode ENABLED - format={shm_frame_format}, slots={shm_slot_count}")
+
+        # ================================================================
+        # PERFORMANCE: Optimized frame capture configuration
+        # ================================================================
+        self.drop_stale_frames = drop_stale_frames
+        self.pin_cpu_affinity = pin_cpu_affinity
+        self.total_workers = total_workers
+        self.buffer_size = buffer_size
+        self.pinned_cores: Optional[List[int]] = None
+
+        # Apply CPU affinity pinning if enabled
+        if pin_cpu_affinity:
+            self.pinned_cores = pin_to_cores(worker_id, total_workers)
+            if self.pinned_cores:
+                self.logger.info(
+                    f"Worker {worker_id}: CPU affinity pinned to cores {self.pinned_cores[0]}-{self.pinned_cores[-1]}"
+                )
+            else:
+                self.logger.warning(f"Worker {worker_id}: CPU affinity pinning failed")
+
+        if drop_stale_frames:
+            self.logger.info(f"Worker {worker_id}: Frame dropping ENABLED (grab/grab/retrieve pattern)")
+
         # ThreadPoolExecutor for I/O-bound frame capture only
         # Encoding is done inline (cv2.imencode releases GIL, ~5ms for 480p)
         #
@@ -197,6 +331,7 @@ class AsyncCameraWorker:
             # Unpack stream_config as keyword arguments (MatriceStream expects **config)
             self.stream = MatriceStream(
                 stream_type=StreamType.REDIS,
+                enable_shm_batching=True,
                 **self.stream_config
             )
 
@@ -271,6 +406,41 @@ class AsyncCameraWorker:
             self._report_health("error", error=str(exc))
             raise
 
+    async def _read_latest_frame(
+        self,
+        cap: cv2.VideoCapture,
+        drop_stale: bool = True
+    ) -> Tuple[bool, Optional[Any]]:
+        """Read latest frame, optionally dropping stale buffered frames.
+
+        This optimization from cv2_bench.py uses grab()/grab()/retrieve()
+        pattern to always get the most recent frame instead of reading
+        stale frames from the buffer.
+
+        Args:
+            cap: OpenCV VideoCapture object
+            drop_stale: If True, use grab/grab/retrieve pattern to skip stale frames
+
+        Returns:
+            Tuple of (success, frame) where frame is None if read failed
+        """
+        loop = asyncio.get_event_loop()
+
+        if drop_stale:
+            # Aggressive frame dropping: grab twice to get latest frame
+            # First grab clears any stale frame, second grab gets current
+            await loop.run_in_executor(self.capture_executor, cap.grab)
+            ret = await loop.run_in_executor(self.capture_executor, cap.grab)
+        else:
+            ret = await loop.run_in_executor(self.capture_executor, cap.grab)
+
+        if not ret:
+            return False, None
+
+        # Retrieve the frame (converts to numpy array)
+        ret, frame = await loop.run_in_executor(self.capture_executor, cap.retrieve)
+        return ret, frame
+
     async def _camera_handler(self, camera_config: Dict[str, Any]):
         """Handle a single camera with async I/O.
 
@@ -278,6 +448,7 @@ class AsyncCameraWorker:
         - Infinite retry with exponential backoff for camera reconnection
         - Video file looping via simulate_video_file_stream parameter
         - Two-level loop: outer (reconnection) + inner (frame processing)
+        - Optimized frame capture with grab/retrieve pattern for latest frame
 
         Args:
             camera_config: Camera configuration dictionary
@@ -339,12 +510,11 @@ class AsyncCameraWorker:
                 # INNER LOOP: Process frames
                 while not self.stop_event.is_set():
                     try:
-                        # Read frame (blocking - run in dedicated capture thread pool)
+                        # Read frame using optimized grab/retrieve pattern
+                        # This gets the latest frame and drops stale buffered frames
                         read_start = time.time()
-                        loop = asyncio.get_event_loop()
-                        ret, frame = await loop.run_in_executor(
-                            self.capture_executor,  # Dedicated thread pool for I/O
-                            cap.read
+                        ret, frame = await self._read_latest_frame(
+                            cap, drop_stale=self.drop_stale_frames
                         )
                         read_time = time.time() - read_start
 
@@ -385,13 +555,23 @@ class AsyncCameraWorker:
                         if width or height:
                             frame = FrameProcessor.resize_frame(frame, width, height)
 
-                        # Process and send frame
-                        await self._process_and_send_frame(
-                            frame, stream_key, stream_group_key, topic,
-                            source, video_props, fps, quality,
-                            actual_width, actual_height, source_type,
-                            frame_counter, camera_location, read_time
-                        )
+                        # ================================================================
+                        # SHM_MODE: Branch based on mode
+                        # ================================================================
+                        if self.use_shm:
+                            await self._process_frame_shm_mode(
+                                frame, stream_key, stream_group_key, topic,
+                                actual_width, actual_height, frame_counter,
+                                camera_location, read_time
+                            )
+                        else:
+                            # EXISTING FLOW: JPEG encode and send full frame
+                            await self._process_and_send_frame(
+                                frame, stream_key, stream_group_key, topic,
+                                source, video_props, fps, quality,
+                                actual_width, actual_height, source_type,
+                                frame_counter, camera_location, read_time
+                            )
 
                         # Maintain target FPS for ALL sources (video files AND live cameras)
                         # This prevents overwhelming the encoder by reading at native camera rate (30+ FPS)
@@ -638,6 +818,185 @@ class AsyncCameraWorker:
             )
             raise
 
+    # ========================================================================
+    # SHM_MODE: Shared Memory Methods
+    # ========================================================================
+
+    def _cleanup_shm_on_exit(self):
+        """Atexit handler to cleanup SHM on unexpected exit/crash.
+
+        CRITICAL: Producer is responsible for unlinking SHM segments.
+        This ensures cleanup happens even on crashes.
+        """
+        for camera_id, shm_buffer in list(self._shm_buffers.items()):
+            try:
+                shm_buffer.close()
+                self.logger.info(f"Worker {self.worker_id}: Cleanup - closed SHM for {camera_id}")
+            except Exception as e:
+                self.logger.warning(f"Worker {self.worker_id}: Failed to cleanup SHM {camera_id}: {e}")
+
+    async def _process_frame_shm_mode(
+        self,
+        frame,
+        stream_key: str,
+        stream_group_key: str,
+        topic: str,
+        width: int,
+        height: int,
+        frame_counter: int,
+        camera_location: str,
+        read_time: float
+    ):
+        """SHM_MODE: Write raw frame to SHM, send metadata to Redis.
+
+        NO JPEG encoding - frame stored as raw NV12 bytes.
+        FrameOptimizer still used to skip similar frames.
+
+        Args:
+            frame: BGR frame from OpenCV
+            stream_key: Camera stream identifier
+            stream_group_key: Stream group identifier
+            topic: Redis stream topic
+            width: Frame width
+            height: Frame height
+            frame_counter: Current frame number
+            camera_location: Camera location string
+            read_time: Time taken to read frame
+        """
+        frame_start = time.time()
+
+        # ================================================================
+        # FRAME OPTIMIZER: Check similarity BEFORE writing to SHM
+        # This saves SHM writes and Redis messages for static scenes
+        # ================================================================
+        is_similar, similarity_score = self.frame_optimizer.is_similar(frame, stream_key)
+        reference_frame_idx = self._last_shm_frame_idx.get(stream_key)
+
+        if is_similar and reference_frame_idx is not None:
+            # Frame is similar - send metadata with reference to previous frame
+            # Consumer can skip reading SHM and use previous result
+            ts_ns = int(time.time() * 1e9)
+            shm_buffer = self._shm_buffers.get(stream_key)
+
+            await self.redis_client.add_shm_metadata(
+                stream_name=topic,
+                cam_id=stream_key,
+                shm_name=shm_buffer.shm_name if shm_buffer else "",
+                frame_idx=reference_frame_idx,  # Reference to cached frame
+                slot=None,  # No new slot written
+                ts_ns=ts_ns,
+                width=width,
+                height=height,
+                format=self.shm_frame_format,
+                is_similar=True,
+                reference_frame_idx=reference_frame_idx,
+                similarity_score=similarity_score,
+                stream_group_key=stream_group_key,
+                camera_location=camera_location,
+                frame_counter=frame_counter,
+            )
+
+            self.statistics.increment_frames_skipped()
+
+            # Track timing (no encoding, minimal write)
+            write_time = time.time() - frame_start - read_time
+            self.statistics.update_timing(stream_key, read_time, write_time, read_time + write_time, 0, 0)
+            return
+
+        # ================================================================
+        # DIFFERENT FRAME: Convert to target format and write to SHM
+        # ================================================================
+
+        # Get or create SHM buffer for this camera
+        shm_buffer = self._get_or_create_shm_buffer(stream_key, width, height)
+
+        # Convert BGR to target format (BGR is default - no conversion needed)
+        convert_start = time.time()
+        if self.shm_frame_format == "RGB":
+            rgb_frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
+            raw_bytes = rgb_frame.tobytes()
+        elif self.shm_frame_format == "NV12":
+            # NV12 conversion - import helper only if needed
+            from matrice_common.stream.shm_ring_buffer import bgr_to_nv12
+            raw_bytes = bgr_to_nv12(frame)
+        else:  # BGR (default) - no conversion needed
+            raw_bytes = frame.tobytes()
+        convert_time = time.time() - convert_start
+
+        # Write to SHM ring buffer
+        frame_idx, slot = shm_buffer.write_frame(raw_bytes)
+
+        # Track this frame_idx for future similar frame references
+        self._last_shm_frame_idx[stream_key] = frame_idx
+
+        # Send metadata-only message to Redis
+        ts_ns = int(time.time() * 1e9)
+        write_start = time.time()
+
+        await self.redis_client.add_shm_metadata(
+            stream_name=topic,
+            cam_id=stream_key,
+            shm_name=shm_buffer.shm_name,
+            frame_idx=frame_idx,
+            slot=slot,
+            ts_ns=ts_ns,
+            width=width,
+            height=height,
+            format=self.shm_frame_format,
+            is_similar=False,
+            stream_group_key=stream_group_key,
+            camera_location=camera_location,
+            frame_counter=frame_counter,
+        )
+
+        write_time = time.time() - write_start
+
+        # Update statistics (no JPEG encoding time, but track format conversion)
+        self.statistics.increment_frames_sent()
+        process_time = read_time + convert_time + write_time
+        self.statistics.update_timing(
+            stream_key, read_time, write_time, process_time,
+            frame_size=len(raw_bytes),
+            encoding_time=convert_time  # Track conversion time in encoding slot
+        )
+
+        # Track total frame time for metrics
+        total_frame_time = time.time() - frame_start
+        self._frame_times.append(total_frame_time)
+
+    def _get_or_create_shm_buffer(self, camera_id: str, width: int, height: int) -> ShmRingBuffer:
+        """Get existing or create new SHM buffer for camera.
+
+        Args:
+            camera_id: Camera stream identifier
+            width: Frame width
+            height: Frame height
+
+        Returns:
+            ShmRingBuffer instance for this camera
+        """
+        if camera_id not in self._shm_buffers:
+            format_map = {
+                "BGR": ShmRingBuffer.FORMAT_BGR,
+                "RGB": ShmRingBuffer.FORMAT_RGB,
+                "NV12": ShmRingBuffer.FORMAT_NV12
+            }
+            frame_format = format_map.get(self.shm_frame_format, ShmRingBuffer.FORMAT_BGR)
+
+            self._shm_buffers[camera_id] = ShmRingBuffer(
+                camera_id=camera_id,
+                width=width,
+                height=height,
+                frame_format=frame_format,
+                slot_count=self.shm_slot_count,
+                create=True  # Producer creates
+            )
+            self.logger.info(
+                f"Worker {self.worker_id}: Created SHM buffer for camera {camera_id}: "
+                f"{width}x{height} {self.shm_frame_format}, {self.shm_slot_count} slots"
+            )
+        return self._shm_buffers[camera_id]
+
     # ========================================================================
     # Dynamic Camera Management Methods
     # ========================================================================
@@ -825,6 +1184,18 @@ class AsyncCameraWorker:
             self.logger.info(f"Worker {self.worker_id}: Released capture {stream_key}")
         self.captures.clear()
 
+        # ================================================================
+        # SHM_MODE: Cleanup and UNLINK SHM buffers (producer responsibility)
+        # ================================================================
+        if self.use_shm:
+            for camera_id, shm_buffer in list(self._shm_buffers.items()):
+                try:
+                    shm_buffer.close()  # This unlinks the SHM segment
+                    self.logger.info(f"Worker {self.worker_id}: Closed and unlinked SHM buffer for camera {camera_id}")
+                except Exception as e:
+                    self.logger.warning(f"Worker {self.worker_id}: Error closing SHM buffer {camera_id}: {e}")
+            self._shm_buffers.clear()
+
         # Close Redis client
         if self.redis_client:
             await self.redis_client.close()
@@ -897,6 +1268,9 @@ class AsyncCameraWorker:
                     'avg_encoding_ms': avg_encoding_ms,
                     'encoding_processes': self.num_encoding_processes,
                     'capture_threads': self.num_capture_threads,
+                    # PERFORMANCE: CPU affinity info
+                    'pinned_cores': self.pinned_cores,
+                    'drop_stale_frames': self.drop_stale_frames,
                 },
                 # Per-camera statistics for metrics reporting
                 'per_camera_stats': per_camera_stats,
@@ -929,7 +1303,20 @@ def run_async_worker(
     stop_event: multiprocessing.Event,
     health_queue: multiprocessing.Queue,
     command_queue: multiprocessing.Queue = None,
-    response_queue: multiprocessing.Queue = None
+    response_queue: multiprocessing.Queue = None,
+    # ================================================================
+    # SHM_MODE: New parameters for shared memory architecture
+    # ================================================================
+    use_shm: bool = True,
+    shm_slot_count: int = 1000,
+    shm_frame_format: str = "BGR",
+    # ================================================================
+    # PERFORMANCE: New parameters for optimized frame capture
+    # ================================================================
+    drop_stale_frames: bool = True,
+    pin_cpu_affinity: bool = True,
+    total_workers: int = 1,
+    buffer_size: int = 1,
 ):
     """Entry point for async worker process.
 
@@ -943,6 +1330,13 @@ def run_async_worker(
         health_queue: Health reporting queue
         command_queue: Queue for receiving dynamic camera commands
         response_queue: Queue for sending command responses
+        use_shm: Enable SHM mode (raw frames in shared memory)
+        shm_slot_count: Number of frame slots per camera ring buffer
+        shm_frame_format: Frame format for SHM storage
+        drop_stale_frames: Use grab()/grab()/retrieve() pattern for latest frame
+        pin_cpu_affinity: Pin worker process to specific CPU cores
+        total_workers: Total number of workers for CPU affinity calculation
+        buffer_size: VideoCapture buffer size (1 = minimal latency)
     """
     # Setup logging for this process
     logging.basicConfig(
@@ -952,7 +1346,19 @@ def run_async_worker(
 
     logger = logging.getLogger(f"AsyncWorker-{worker_id}")
     logger.info(f"Starting async worker {worker_id}")
-
+    import os
+    os.environ["OMP_NUM_THREADS"] = "1"
+    os.environ["KMP_NUM_THREADS"] = "1"
+    os.environ["KMP_BLOCKTIME"] = "0"
+    os.environ["TBB_NUM_THREADS"] = "1"
+    os.environ["OPENBLAS_NUM_THREADS"] = "1"
+    os.environ["MKL_NUM_THREADS"] = "1"
+    os.environ["VECLIB_MAXIMUM_THREADS"] = "1"
+    os.environ["NUMEXPR_NUM_THREADS"] = "1"
+    import cv2
+    cv2.setNumThreads(1)
+    cv2.setUseOptimized(True)
+    cv2.ocl.setUseOpenCL(False)
     try:
         # Create worker
         worker = AsyncCameraWorker(
@@ -962,7 +1368,16 @@ def run_async_worker(
             stop_event=stop_event,
             health_queue=health_queue,
             command_queue=command_queue,
-            response_queue=response_queue
+            response_queue=response_queue,
+            # SHM_MODE: Pass through shared memory parameters
+            use_shm=use_shm,
+            shm_slot_count=shm_slot_count,
+            shm_frame_format=shm_frame_format,
+            # PERFORMANCE: Pass through optimized frame capture parameters
+            drop_stale_frames=drop_stale_frames,
+            pin_cpu_affinity=pin_cpu_affinity,
+            total_workers=total_workers,
+            buffer_size=buffer_size,
         )
 
         # Run event loop
