@@ -48,6 +48,7 @@ class GStreamerConfig:
     - Simple frame caching (hash-based works perfectly)
     - Maximum throughput for static scenes
     """
+    # Basic encoder settings
     encoder: str = "jpeg"  # Default to JPEG for frame-by-frame (was "auto")
     codec: str = "h264"  # h264, h265 (only used for video encoders)
     bitrate: int = 4000000  # 4 Mbps (only used for video encoders)
@@ -56,6 +57,17 @@ class GStreamerConfig:
     use_cuda_memory: bool = True
     jpeg_quality: int = 85  # JPEG quality (1-100, higher=better quality)
     gop_size: int = 30  # I-frame interval (only used for video encoders)
+
+    # Platform-specific settings (NEW)
+    platform: str = "auto"  # auto, jetson, desktop-gpu, cpu, intel, amd
+    enable_platform_override: bool = True  # Allow manual platform override
+    use_hardware_decode: bool = True  # Use nvv4l2decoder, nvdec, vaapi for decode
+    use_hardware_jpeg: bool = True  # Use nvjpegenc (Jetson), vaapijpegenc (Intel/AMD)
+    jetson_use_nvmm: bool = True  # NVMM zero-copy memory on Jetson
+    frame_optimizer_mode: str = "hash-only"  # hash-only, dual-appsink, disabled
+    source_optimization: bool = True  # Enable source-specific optimizations
+    fallback_on_error: bool = True  # Graceful fallback to CPU if HW fails
+    verbose_pipeline_logging: bool = False  # Debug pipeline construction
     
 
 @dataclass  
@@ -90,12 +102,12 @@ class GStreamerPipeline:
         config: GStreamerConfig,
     ):
         """Initialize GStreamer pipeline.
-        
+
         Args:
             stream_key: Unique identifier for this stream
             source: Video source (camera index, file path, RTSP URL, etc.)
             width: Target output width
-            height: Target output height  
+            height: Target output height
             fps: Target frames per second
             config: GStreamer configuration
         """
@@ -105,23 +117,31 @@ class GStreamerPipeline:
         self.height = height
         self.fps = fps
         self.config = config
-        
+
         self.logger = logging.getLogger(f"GStreamerPipeline-{stream_key}")
-        
+
         self.pipeline = None
         self.appsink = None
         self.running = False
-        
+
         # PTS-based latency tracking
         self.start_pts: Optional[int] = None
         self.wall_start_time: float = 0.0
-        
+
         # Metrics
         self.metrics = GStreamerMetrics()
         self._metrics_lock = threading.Lock()
-        
+
         # Detected encoder type
         self._encoder_type: Optional[str] = None
+
+        # Platform detection and pipeline builder (NEW)
+        from .device_detection import PlatformDetector
+        from .platform_pipelines import PipelineFactory
+
+        self.platform_detector = PlatformDetector.get_instance()
+        self.platform_info = self.platform_detector.detect()
+        self.pipeline_builder = PipelineFactory.get_builder(config, self.platform_info)
         
     def _detect_encoder(self) -> str:
         """Detect the best available encoder."""
@@ -275,102 +295,110 @@ class GStreamerPipeline:
         return encoder_settings, caps_out
         
     def _build_pipeline_string(self) -> str:
-        """Build complete GStreamer pipeline string."""
-        source_element = self._build_source_element()
-        encoder_settings, caps_out = self._build_encoder_element()
-        
-        # Determine input format based on encoder
-        input_format = "NV12" if self._encoder_type == "nvenc" else "I420"
-        
-        # Build pipeline with appropriate elements
-        if self.config.use_cuda_memory and self._encoder_type == "nvenc":
-            # Zero-copy CUDA pipeline
-            pipeline = (
-                f"{source_element} ! "
-                f"videoconvert ! "
-                f"videoscale ! "
-                f"video/x-raw,format=NV12,width={self.width},height={self.height},"
-                f"framerate={self.fps}/1 ! "
-                f"queue max-size-buffers=1 leaky=downstream ! "
-                f"cudaupload cuda-device-id={self.config.gpu_id} ! "
-                f"video/x-raw(memory:CUDAMemory),format=NV12 ! "
-                f"{encoder_settings}"
-                f"! "
-                f"{caps_out} ! "
-                f"queue max-size-buffers=1 leaky=downstream ! "
-                f"appsink name=sink emit-signals=false sync=false async=false "
-                f"max-buffers=1 drop=true enable-last-sample=false"
-            )
+        """Build platform-optimized GStreamer pipeline string."""
+        # Detect source type
+        source_type = self._detect_source_type()
+
+        # Detect encoder (for compatibility)
+        encoder = self._detect_encoder()
+        self._encoder_type = encoder
+
+        # Build config dict for pipeline builder
+        builder_config = {
+            'source_type': source_type,
+            'source': self.source,
+            'width': self.width,
+            'height': self.height,
+            'fps': self.fps,
+            'encoder': encoder,
+            'quality': self.config.jpeg_quality,
+            'bitrate': self.config.bitrate,
+        }
+
+        # Use pipeline builder to construct platform-optimized pipeline
+        if self.config.frame_optimizer_mode == "dual-appsink":
+            pipeline_str = self.pipeline_builder.build_dual_appsink_pipeline(builder_config)
         else:
-            # Standard CPU pipeline
-            # Note: videorate ensures correct FPS, videoscale handles resolution
-            # Using capsfilter without strict framerate for better compatibility
-            pipeline = (
-                f"{source_element} ! "
-                f"videoconvert ! "
-                f"videoscale ! "
-                f"video/x-raw,format={input_format},width={self.width},height={self.height} ! "
-                f"videorate ! "
-                f"video/x-raw,framerate={self.fps}/1 ! "
-                f"queue max-size-buffers=2 leaky=downstream ! "
-                f"{encoder_settings}"
-                f"! "
-                f"{caps_out} ! "
-                f"queue max-size-buffers=2 leaky=downstream ! "
-                f"appsink name=sink emit-signals=false sync=false async=false "
-                f"max-buffers=2 drop=true enable-last-sample=false"
-            )
-            
-        return pipeline
+            pipeline_str = self.pipeline_builder.build_complete_pipeline(builder_config)
+
+        if self.config.verbose_pipeline_logging:
+            self.logger.info(f"Platform: {self.platform_info.platform_type.value}")
+            self.logger.info(f"Encoder: {encoder}, Decoder: {self.platform_info.recommended_decoder}")
+            self.logger.info(f"Pipeline: {pipeline_str[:200]}...")
+
+        return pipeline_str
         
     def start(self) -> bool:
         """Start the GStreamer pipeline.
-        
+
         Returns:
             bool: True if started successfully
         """
         if not GST_AVAILABLE:
             raise RuntimeError("GStreamer not available")
-            
+
         if self.running:
             self.logger.warning("Pipeline already running")
             return False
-            
+
         try:
             # Build and parse pipeline
             pipeline_str = self._build_pipeline_string()
             self.logger.info(f"Creating pipeline: {pipeline_str[:200]}...")
-            
+
             self.pipeline = Gst.parse_launch(pipeline_str)
-            
-            # Get appsink
+
+            # Get appsink (or appsinks for dual-appsink mode)
             self.appsink = self.pipeline.get_by_name("sink")
+            if not self.appsink:
+                # Try dual-appsink mode naming
+                self.appsink = self.pipeline.get_by_name("output-sink")
             if not self.appsink:
                 raise RuntimeError("Failed to get appsink from pipeline")
-                
+
             # Setup bus for error handling
             bus = self.pipeline.get_bus()
             bus.add_signal_watch()
             bus.connect("message::error", self._on_error)
             bus.connect("message::warning", self._on_warning)
             bus.connect("message::eos", self._on_eos)
-            
+
             # Start pipeline
             ret = self.pipeline.set_state(Gst.State.PLAYING)
             if ret == Gst.StateChangeReturn.FAILURE:
                 raise RuntimeError("Failed to start pipeline")
-                
+
             # Wait for pipeline to be ready
             self.pipeline.get_state(Gst.CLOCK_TIME_NONE)
-            
+
             self.running = True
             self.metrics.start_time = time.time()
-            
-            self.logger.info(f"Pipeline started with encoder: {self._encoder_type}")
+
+            self.logger.info(
+                f"Pipeline started - Platform: {self.platform_info.platform_type.value}, "
+                f"Encoder: {self._encoder_type}"
+            )
             return True
-            
+
         except Exception as e:
             self.logger.error(f"Failed to start pipeline: {e}")
+
+            # Fallback to CPU-only pipeline if enabled
+            if self.config.fallback_on_error and self.config.platform != "cpu":
+                self.logger.warning("Attempting fallback to CPU-only pipeline...")
+
+                try:
+                    # Force CPU-only pipeline
+                    self.config.platform = "cpu"
+                    from .platform_pipelines import PipelineFactory, CpuOnlyPipelineBuilder
+                    self.pipeline_builder = CpuOnlyPipelineBuilder(self.config, self.platform_info)
+
+                    # Retry with CPU pipeline
+                    return self.start()
+
+                except Exception as fallback_error:
+                    self.logger.error(f"Fallback to CPU also failed: {fallback_error}")
+
             self.stop()
             raise
             
