@@ -54,9 +54,17 @@ class GStreamerAsyncWorker:
         gstreamer_codec: str = "h264",
         gstreamer_preset: str = "low-latency",
         gpu_id: int = 0,
+        # Platform-specific parameters
+        platform: str = "auto",
+        use_hardware_decode: bool = True,
+        use_hardware_jpeg: bool = True,
+        jetson_use_nvmm: bool = True,
+        frame_optimizer_mode: str = "hash-only",
+        fallback_on_error: bool = True,
+        verbose_pipeline_logging: bool = False,
     ):
         """Initialize GStreamer async worker.
-        
+
         Args:
             worker_id: Unique identifier for this worker
             camera_configs: List of camera configurations
@@ -69,6 +77,13 @@ class GStreamerAsyncWorker:
             gstreamer_codec: Codec (h264, h265)
             gstreamer_preset: NVENC preset
             gpu_id: GPU device ID for NVENC
+            platform: Platform override (auto, jetson, desktop-gpu, intel, amd, cpu)
+            use_hardware_decode: Enable hardware decode
+            use_hardware_jpeg: Enable hardware JPEG encoding
+            jetson_use_nvmm: Use NVMM zero-copy on Jetson
+            frame_optimizer_mode: Frame optimization mode
+            fallback_on_error: Fallback to CPU on errors
+            verbose_pipeline_logging: Enable verbose logging
         """
         if not GST_AVAILABLE:
             raise RuntimeError("GStreamer not available for GStreamerAsyncWorker")
@@ -90,10 +105,39 @@ class GStreamerAsyncWorker:
         # Logging
         self.logger = logging.getLogger(f"GStreamerWorker-{worker_id}")
         self.logger.info(f"Initializing GStreamer worker {worker_id} with {len(camera_configs)} cameras")
-        
+
         # Initialize GStreamer
         Gst.init(None)
-        
+
+        # Platform detection and pipeline builder (NEW)
+        from .device_detection import PlatformDetector
+        from .platform_pipelines import PipelineFactory
+        from .gstreamer_camera_streamer import GStreamerConfig
+
+        # Build GStreamerConfig from worker settings
+        self.gstreamer_config = GStreamerConfig(
+            encoder=gstreamer_encoder,
+            codec=gstreamer_codec,
+            preset=gstreamer_preset,
+            gpu_id=gpu_id,
+            platform=platform,
+            use_hardware_decode=use_hardware_decode,
+            use_hardware_jpeg=use_hardware_jpeg,
+            jetson_use_nvmm=jetson_use_nvmm,
+            frame_optimizer_mode=frame_optimizer_mode,
+            fallback_on_error=fallback_on_error,
+            verbose_pipeline_logging=verbose_pipeline_logging,
+        )
+
+        self.platform_detector = PlatformDetector.get_instance()
+        self.platform_info = self.platform_detector.detect()
+        self.pipeline_builder = PipelineFactory.get_builder(self.gstreamer_config, self.platform_info)
+
+        self.logger.info(
+            f"Worker {worker_id}: Platform={self.platform_info.platform_type.value}, "
+            f"Recommended encoder={self.platform_info.recommended_encoder}"
+        )
+
         # Components
         self.message_builder = StreamMessageBuilder(
             service_id=stream_config.get('service_id', 'streaming_gateway'),
@@ -133,7 +177,7 @@ class GStreamerAsyncWorker:
 
         # Detected encoder
         self._detected_encoder: Optional[str] = None
-        
+
         self.logger.info(
             f"GStreamer Worker {worker_id}: encoder={gstreamer_encoder}, "
             f"codec={gstreamer_codec}, gpu_id={gpu_id}"
@@ -171,104 +215,57 @@ class GStreamerAsyncWorker:
         fps: int,
         quality: int = 85
     ) -> str:
-        """Build GStreamer pipeline string for a camera.
-        
+        """Build platform-optimized GStreamer pipeline string.
+
         Args:
             source: Video source
             width: Target width
             height: Target height
             fps: Target FPS
             quality: JPEG quality (for jpeg encoder)
-            
+
         Returns:
             Pipeline description string
         """
-        encoder_type = self._detected_encoder or self._detect_encoder()
-        self._detected_encoder = encoder_type
-        
-        # Build source element
+        # Detect source type
         source_str = str(source)
         if isinstance(source, int):
-            source_elem = f"v4l2src device=/dev/video{source}"
+            source_type = "camera"
         elif source_str.startswith("rtsp://"):
-            source_elem = f"rtspsrc location={source} latency=100 ! rtph264depay ! h264parse ! avdec_h264"
+            source_type = "rtsp"
         elif source_str.startswith(("http://", "https://")):
-            source_elem = f"souphttpsrc location={source} ! qtdemux ! avdec_h264"
-        elif source_str.endswith((".mp4", ".mov", ".m4v")):
-            # Use qtdemux for mp4/mov files (avoids dynamic pad issues with decodebin)
-            source_elem = f"filesrc location={source} ! qtdemux ! avdec_h264"
-        elif source_str.endswith((".mkv", ".webm")):
-            source_elem = f"filesrc location={source} ! matroskademux ! avdec_h264"
-        elif source_str.endswith(".avi"):
-            source_elem = f"filesrc location={source} ! avidemux ! avdec_h264"
+            source_type = "http"
+        elif source_str.endswith((".mp4", ".mov", ".m4v", ".mkv", ".webm", ".avi")):
+            source_type = "file"
         else:
-            source_elem = "videotestsrc pattern=smpte is-live=true"
-            
-        # Build encoder element
-        bitrate_kbps = 4000
-        
-        if encoder_type == "nvenc":
-            if self.gstreamer_codec == "h265":
-                enc = "nvh265enc"
-            else:
-                enc = "nvh264enc"
-            encoder_elem = (
-                f"{enc} "
-                f"cuda-device-id={self.gpu_id} "
-                f"preset={self.gstreamer_preset} "
-                f"bitrate={bitrate_kbps} "
-                f"gop-size=30 "
-                f"zerolatency=true "
-                f"bframes=0 "
-            )
-            caps_out = f"video/x-{self.gstreamer_codec},profile=main"
-            input_format = "NV12"
-            
-        elif encoder_type == "x264":
-            encoder_elem = (
-                f"x264enc "
-                f"speed-preset=ultrafast "
-                f"tune=zerolatency "
-                f"bitrate={bitrate_kbps} "
-                f"key-int-max=30 "
-                f"bframes=0 "
-                f"threads=2 "
-            )
-            caps_out = "video/x-h264,profile=baseline"
-            input_format = "I420"
-            
-        elif encoder_type == "openh264":
-            encoder_elem = (
-                f"openh264enc "
-                f"bitrate={bitrate_kbps * 1000} "
-                f"complexity=low "
-            )
-            caps_out = "video/x-h264,profile=baseline"
-            input_format = "I420"
-            
-        else:  # jpeg
-            encoder_elem = f"jpegenc quality={quality} idct-method=ifast"
-            caps_out = "image/jpeg"
-            input_format = "I420"
-            
-        # Build complete pipeline
-        # Note: Split videoscale and videorate caps for better compatibility
-        pipeline = (
-            f"{source_elem} ! "
-            f"videoconvert ! "
-            f"videoscale ! "
-            f"video/x-raw,format={input_format},width={width},height={height} ! "
-            f"videorate ! "
-            f"video/x-raw,framerate={fps}/1 ! "
-            f"queue max-size-buffers=2 leaky=downstream ! "
-            f"{encoder_elem} ! "
-            f"{caps_out} ! "
-            f"queue max-size-buffers=2 leaky=downstream ! "
-            f"appsink name=sink emit-signals=false sync=false async=false "
-            f"max-buffers=2 drop=true enable-last-sample=false"
-        )
-        
-        return pipeline
+            source_type = "camera"
+
+        # Detect encoder
+        encoder_type = self._detected_encoder or self._detect_encoder()
+        self._detected_encoder = encoder_type
+
+        # Build config dict for pipeline builder
+        builder_config = {
+            'source_type': source_type,
+            'source': source,
+            'width': width,
+            'height': height,
+            'fps': fps,
+            'encoder': encoder_type,
+            'quality': quality,
+            'bitrate': 4000000,  # 4 Mbps
+        }
+
+        # Use pipeline builder to construct platform-optimized pipeline
+        if self.gstreamer_config.frame_optimizer_mode == "dual-appsink":
+            pipeline_str = self.pipeline_builder.build_dual_appsink_pipeline(builder_config)
+        else:
+            pipeline_str = self.pipeline_builder.build_complete_pipeline(builder_config)
+
+        if self.gstreamer_config.verbose_pipeline_logging:
+            self.logger.info(f"Worker {self.worker_id} Pipeline: {pipeline_str[:150]}...")
+
+        return pipeline_str
         
     async def initialize(self):
         """Initialize async resources."""
@@ -993,9 +990,16 @@ def run_gstreamer_worker(
     gstreamer_codec: str = "h264",
     gstreamer_preset: str = "low-latency",
     gpu_id: int = 0,
+    platform: str = "auto",
+    use_hardware_decode: bool = True,
+    use_hardware_jpeg: bool = True,
+    jetson_use_nvmm: bool = True,
+    frame_optimizer_mode: str = "hash-only",
+    fallback_on_error: bool = True,
+    verbose_pipeline_logging: bool = False,
 ):
     """Entry point for GStreamer worker process.
-    
+
     Args:
         worker_id: Worker identifier
         camera_configs: Camera configurations
@@ -1008,6 +1012,13 @@ def run_gstreamer_worker(
         gstreamer_codec: Codec
         gstreamer_preset: NVENC preset
         gpu_id: GPU device ID
+        platform: Platform override
+        use_hardware_decode: Enable hardware decode
+        use_hardware_jpeg: Enable hardware JPEG
+        jetson_use_nvmm: Use NVMM on Jetson
+        frame_optimizer_mode: Frame optimization mode
+        fallback_on_error: Fallback to CPU on errors
+        verbose_pipeline_logging: Verbose logging
     """
     logging.basicConfig(
         level=logging.INFO,
@@ -1030,6 +1041,13 @@ def run_gstreamer_worker(
             gstreamer_codec=gstreamer_codec,
             gstreamer_preset=gstreamer_preset,
             gpu_id=gpu_id,
+            platform=platform,
+            use_hardware_decode=use_hardware_decode,
+            use_hardware_jpeg=use_hardware_jpeg,
+            jetson_use_nvmm=jetson_use_nvmm,
+            frame_optimizer_mode=frame_optimizer_mode,
+            fallback_on_error=fallback_on_error,
+            verbose_pipeline_logging=verbose_pipeline_logging,
         )
         
         asyncio.run(worker.run())
