@@ -34,9 +34,17 @@ class GStreamerWorkerManager:
         gstreamer_codec: str = "h264",
         gstreamer_preset: str = "low-latency",
         gpu_id: int = 0,
+        # Platform-specific parameters
+        platform: str = "auto",
+        use_hardware_decode: bool = True,
+        use_hardware_jpeg: bool = True,
+        jetson_use_nvmm: bool = True,
+        frame_optimizer_mode: str = "hash-only",
+        fallback_on_error: bool = True,
+        verbose_pipeline_logging: bool = False,
     ):
         """Initialize GStreamer worker manager.
-        
+
         Args:
             camera_configs: List of all camera configurations
             stream_config: Streaming configuration (Redis, Kafka, etc.)
@@ -47,6 +55,13 @@ class GStreamerWorkerManager:
             gstreamer_codec: Codec (h264, h265)
             gstreamer_preset: NVENC preset
             gpu_id: GPU device ID for NVENC
+            platform: Platform override (auto, jetson, desktop-gpu, intel, amd, cpu)
+            use_hardware_decode: Enable hardware decode
+            use_hardware_jpeg: Enable hardware JPEG encoding
+            jetson_use_nvmm: Use NVMM zero-copy on Jetson
+            frame_optimizer_mode: Frame optimization mode (hash-only, dual-appsink, disabled)
+            fallback_on_error: Fallback to CPU pipeline on errors
+            verbose_pipeline_logging: Enable verbose pipeline logging
         """
         if not is_gstreamer_available():
             raise RuntimeError(
@@ -60,26 +75,62 @@ class GStreamerWorkerManager:
         self.gstreamer_codec = gstreamer_codec
         self.gstreamer_preset = gstreamer_preset
         self.gpu_id = gpu_id
-        
-        # Calculate worker count
+        # Platform-specific settings
+        self.platform = platform
+        self.use_hardware_decode = use_hardware_decode
+        self.use_hardware_jpeg = use_hardware_jpeg
+        self.jetson_use_nvmm = jetson_use_nvmm
+        self.frame_optimizer_mode = frame_optimizer_mode
+        self.fallback_on_error = fallback_on_error
+        self.verbose_pipeline_logging = verbose_pipeline_logging
+
+        # Platform detection (NEW)
+        from .device_detection import PlatformDetector, PlatformType
+        self.platform_detector = PlatformDetector.get_instance()
+        self.platform_info = self.platform_detector.detect()
+
+        self.logger = logging.getLogger(__name__)
+        self.logger.info(
+            f"Detected platform: {self.platform_info.platform_type.value}, "
+            f"Model: {self.platform_info.model}"
+        )
+
+        # Platform-aware worker calculation (NEW)
         if num_workers is None:
             cpu_count = os.cpu_count() or 4
             num_cameras = len(camera_configs)
-            
-            if cpu_count >= 16 or num_cameras >= 100:
-                # For large deployments, use camera-based calculation
+
+            # Adjust based on platform capabilities
+            if self.platform_info.platform_type == PlatformType.JETSON:
+                # Jetson has fewer cores, limit workers
+                # Hardware encoding means each worker can handle more cameras
+                target_per_worker = 30  # Jetson HW encoding is efficient
+                calculated = max(4, min(num_cameras // target_per_worker, 8))
+                self.logger.info(f"Jetson platform: limiting to max 8 workers")
+
+            elif self.platform_info.platform_type in (PlatformType.INTEL_GPU, PlatformType.AMD_GPU):
+                # VAAPI acceleration, moderate parallelism
                 target_per_worker = 25
-                calculated = max(4, min(num_cameras // target_per_worker, 50))
-            else:
+                calculated = max(4, min(num_cameras // target_per_worker, 16))
+
+            elif self.platform_info.platform_type == PlatformType.DESKTOP_NVIDIA_GPU:
+                # Desktop GPU, good parallelism
+                if cpu_count >= 16 or num_cameras >= 100:
+                    target_per_worker = 25
+                    calculated = max(4, min(num_cameras // target_per_worker, 50))
+                else:
+                    calculated = max(4, int(cpu_count * cpu_percentage))
+
+            else:  # CPU-only
+                # CPU-only needs more workers for parallelism
                 calculated = max(4, int(cpu_count * cpu_percentage))
-                
+
             self.num_workers = min(calculated, num_cameras) if num_cameras > 0 else calculated
         else:
             self.num_workers = num_workers
-            
+
         self.max_cameras_per_worker = max_cameras_per_worker
-        
-        self.logger = logging.getLogger(__name__)
+
         self.logger.info(
             f"GStreamerWorkerManager: {self.num_workers} workers for {len(camera_configs)} cameras, "
             f"encoder={gstreamer_encoder}, codec={gstreamer_codec}, gpu={gpu_id}"
@@ -193,6 +244,13 @@ class GStreamerWorkerManager:
                     self.gstreamer_codec,
                     self.gstreamer_preset,
                     self.gpu_id,
+                    self.platform,
+                    self.use_hardware_decode,
+                    self.use_hardware_jpeg,
+                    self.jetson_use_nvmm,
+                    self.frame_optimizer_mode,
+                    self.fallback_on_error,
+                    self.verbose_pipeline_logging,
                 ),
                 name=f"GStreamerWorker-{worker_id}",
                 daemon=False
