@@ -16,6 +16,7 @@ from .async_camera_worker import run_async_worker
 from .encoding_pool_manager import EncodingPoolManager
 from .camera_streamer import CameraStreamer
 
+USE_SHM = os.getenv("USE_SHM", "false").lower() == "true"
 
 class WorkerManager:
     """Manages multiple async camera worker processes with dynamic scaling.
@@ -32,7 +33,19 @@ class WorkerManager:
         num_workers: Optional[int] = None,
         cpu_percentage: float = 0.9,
         num_encoding_workers: Optional[int] = None,
-        max_cameras_per_worker: int = 100
+        max_cameras_per_worker: int = 100,
+        # ================================================================
+        # SHM_MODE: New parameters for shared memory architecture
+        # ================================================================
+        use_shm: bool = USE_SHM,  # Enable SHM mode (raw frames in shared memory)
+        shm_slot_count: int = 1000,  # Ring buffer size per camera (increased for consumer lag)
+        shm_frame_format: str = "BGR",  # Frame format: "BGR", "RGB", or "NV12"
+        # ================================================================
+        # PERFORMANCE: New parameters for optimized frame capture
+        # ================================================================
+        drop_stale_frames: bool = True,   # Use grab()/grab()/retrieve() for latest frame
+        pin_cpu_affinity: bool = True,    # Pin workers to specific CPU cores
+        buffer_size: int = 1,             # VideoCapture buffer size (1 = minimal latency)
     ):
         """Initialize worker manager with dynamic CPU-based scaling.
 
@@ -43,6 +56,12 @@ class WorkerManager:
             cpu_percentage: Percentage of CPU cores to use (default: 0.9 = 90%)
             num_encoding_workers: Number of encoding workers (default: CPU_count - 2)
             max_cameras_per_worker: Maximum cameras per worker for load balancing (default: 100)
+            use_shm: Enable SHM mode (raw frames in shared memory, metadata in Redis)
+            shm_slot_count: Number of frame slots per camera ring buffer
+            shm_frame_format: Frame format for SHM storage
+            drop_stale_frames: Use grab()/grab()/retrieve() pattern for latest frame
+            pin_cpu_affinity: Pin worker processes to specific CPU cores for cache locality
+            buffer_size: VideoCapture buffer size (1 = minimal latency)
         """
         self.camera_configs = camera_configs
         self.stream_config = stream_config
@@ -84,6 +103,33 @@ class WorkerManager:
             f"for {len(camera_configs)} cameras"
         )
 
+        # ================================================================
+        # SHM_MODE: Store shared memory configuration
+        # ================================================================
+        self.use_shm = use_shm
+        self.shm_slot_count = shm_slot_count
+        self.shm_frame_format = shm_frame_format
+
+        if use_shm:
+            self.logger.info(
+                f"SHM_MODE ENABLED: format={shm_frame_format}, slots={shm_slot_count}"
+            )
+
+        # ================================================================
+        # PERFORMANCE: Store optimized frame capture configuration
+        # ================================================================
+        self.drop_stale_frames = drop_stale_frames
+        self.pin_cpu_affinity = pin_cpu_affinity
+        self.buffer_size = buffer_size
+
+        if drop_stale_frames or pin_cpu_affinity:
+            self.logger.info(
+                f"PERFORMANCE OPTIMIZATIONS ENABLED: "
+                f"drop_stale_frames={drop_stale_frames}, "
+                f"pin_cpu_affinity={pin_cpu_affinity}, "
+                f"buffer_size={buffer_size}"
+            )
+
         # Note: Batch parameters are calculated per-worker in _start_worker()
         # based on each worker's camera count, not the global total.
         # This ensures optimal batching when cameras are distributed across workers.
@@ -235,7 +281,16 @@ class WorkerManager:
                     self.stop_event,
                     self.health_queue,
                     command_queue,        # Pass command queue for dynamic camera ops
-                    self.response_queue   # Pass response queue for acknowledgments
+                    self.response_queue,  # Pass response queue for acknowledgments
+                    # SHM_MODE: Pass shared memory parameters
+                    self.use_shm,
+                    self.shm_slot_count,
+                    self.shm_frame_format,
+                    # PERFORMANCE: Pass optimized frame capture parameters
+                    self.drop_stale_frames,
+                    self.pin_cpu_affinity,
+                    self.num_workers,     # Total workers for CPU affinity calculation
+                    self.buffer_size,
                 ),
                 name=f"AsyncWorker-{worker_id}",
                 daemon=False  # Non-daemon so we can properly wait for shutdown
