@@ -544,11 +544,15 @@ class DynamicCameraManager:
                 simulate_video = True
                 
                 # Try to get signed URL if we have gateway_util
-                if self.gateway_util and source:
+                # Always fetch presigned URL for FILE cameras - the API uses camera_id,
+                # not the local simulationVideoPath, to look up and sign the URL
+                if self.gateway_util:
                     try:
                         stream_url_data = self.gateway_util.get_simulated_stream_url(camera_id)
                         if stream_url_data and stream_url_data.get('url'):
                             source = stream_url_data['url']
+                        else:
+                            self.logger.warning(f"No signed URL returned for FILE camera {camera_id}, using raw path")
                     except Exception as e:
                         self.logger.warning(f"Failed to get signed URL for camera {camera_id}: {e}")
             
@@ -1105,11 +1109,15 @@ class DynamicCameraManagerForWorkers:
                 simulate_video = True
 
                 # Try to get signed URL if we have gateway_util
-                if self.gateway_util and source:
+                # Always fetch presigned URL for FILE cameras - the API uses camera_id,
+                # not the local simulationVideoPath, to look up and sign the URL
+                if self.gateway_util:
                     try:
                         stream_url_data = self.gateway_util.get_simulated_stream_url(camera_id)
                         if stream_url_data and stream_url_data.get('url'):
                             source = stream_url_data['url']
+                        else:
+                            self.logger.warning(f"No signed URL returned for FILE camera {camera_id}, using raw path")
                     except Exception as e:
                         self.logger.warning(f"Failed to get signed URL for camera {camera_id}: {e}")
 
