@@ -707,6 +707,10 @@ class MetricsManager:
     calculates statistics, and reports them via Kafka.
     """
 
+    # ANSI escape codes for BOLD text in terminal
+    BOLD = "\033[1m"
+    RESET = "\033[0m"
+
     def __init__(
         self,
         streaming_gateway,
@@ -743,6 +747,8 @@ class MetricsManager:
         self.last_log_time = 0
         self.last_metrics_log_time = 0
         self.metrics_log_interval = 60.0  # Log metrics summary every 60 seconds
+        self.last_aggregate_log_time = 0
+        self.aggregate_log_interval = 60.0  # Log aggregate metrics with BOLD every 60 seconds
         self.enabled = True
 
         flow_type = "async_workers" if self.use_async_workers else "camera_streamer"
@@ -771,6 +777,11 @@ class MetricsManager:
                 self._log_metrics_summary(snapshot)
                 self.last_metrics_log_time = current_time
 
+            # Log aggregate metrics with BOLD every minute
+            if current_time - self.last_aggregate_log_time >= self.aggregate_log_interval:
+                self._log_aggregate_metrics_bold()
+                self.last_aggregate_log_time = current_time
+
             # Report if interval has elapsed
             if current_time - self.last_report_time >= self.config.reporting_interval:
                 self._generate_and_send_report()
@@ -792,6 +803,222 @@ class MetricsManager:
         except Exception as e:
             logging.warning(f"Error logging metrics summary: {e}")
 
+    def _log_aggregate_metrics_bold(self):
+        """Log comprehensive aggregate metrics with BOLD formatting every minute.
+
+        Reports:
+        - Overall AVG FPS across all cameras
+        - Latency breakdown (read, encode/convert, write averages)
+        - Total throughput (sum of all cameras' FPS)
+        - Total data throughput (KB/s)
+        """
+        try:
+            gateway_stats = self.streaming_gateway.get_statistics()
+
+            if self.use_async_workers:
+                self._log_aggregate_async_workers_bold(gateway_stats)
+            else:
+                self._log_aggregate_camera_streamer_bold(gateway_stats)
+
+        except Exception as e:
+            logging.warning(f"Error logging aggregate metrics: {e}")
+
+    def _log_aggregate_async_workers_bold(self, gateway_stats: Dict[str, Any]):
+        """Log aggregate metrics for async workers with BOLD formatting."""
+        worker_stats = gateway_stats.get("worker_stats", {})
+        stream_keys = gateway_stats.get("my_stream_keys", [])
+        runtime = gateway_stats.get("runtime_seconds", 0)
+
+        num_workers = worker_stats.get("num_workers", 0)
+        running_workers = worker_stats.get("running_workers", 0)
+        total_cameras = worker_stats.get("total_cameras", len(stream_keys))
+        per_camera_stats = worker_stats.get("per_camera_stats", {})
+
+        # Detect SHM mode from worker_manager
+        use_shm = False
+        shm_format = "N/A"
+        worker_manager = getattr(self.streaming_gateway, 'worker_manager', None)
+        if worker_manager:
+            use_shm = getattr(worker_manager, 'use_shm', False)
+            shm_format = getattr(worker_manager, 'shm_frame_format', 'N/A') if use_shm else "N/A"
+
+        # Aggregate FPS stats
+        total_fps = 0.0
+        fps_values = []
+        for stream_key, stats in per_camera_stats.items():
+            fps_avg = stats.get("fps", {}).get("avg", 0)
+            if fps_avg > 0:
+                total_fps += fps_avg
+                fps_values.append(fps_avg)
+
+        avg_fps = sum(fps_values) / len(fps_values) if fps_values else 0
+        min_fps = min(fps_values) if fps_values else 0
+        max_fps = max(fps_values) if fps_values else 0
+
+        # Aggregate latency stats (in ms)
+        read_times = []
+        write_times = []
+        encoding_times = []
+
+        for stream_key, stats in per_camera_stats.items():
+            read_ms = stats.get("read_time_ms", {}).get("avg", 0)
+            write_ms = stats.get("write_time_ms", {}).get("avg", 0)
+            encoding_ms = stats.get("encoding_time_ms", {}).get("avg", 0)
+
+            if read_ms > 0:
+                read_times.append(read_ms)
+            if write_ms > 0:
+                write_times.append(write_ms)
+            if encoding_ms > 0:
+                encoding_times.append(encoding_ms)
+
+        avg_read_ms = sum(read_times) / len(read_times) if read_times else 0
+        avg_write_ms = sum(write_times) / len(write_times) if write_times else 0
+        avg_encoding_ms = sum(encoding_times) / len(encoding_times) if encoding_times else 0
+        total_latency_ms = avg_read_ms + avg_encoding_ms + avg_write_ms
+
+        # Aggregate frame size and throughput
+        total_frame_size_kb = 0.0
+        frame_sizes = []
+        for stream_key, stats in per_camera_stats.items():
+            frame_size_bytes = stats.get("frame_size_bytes", {}).get("avg", 0)
+            if frame_size_bytes > 0:
+                frame_sizes.append(frame_size_bytes)
+                total_frame_size_kb += frame_size_bytes / 1024
+
+        avg_frame_size_kb = (sum(frame_sizes) / len(frame_sizes) / 1024) if frame_sizes else 0
+
+        # Total throughput: sum of (FPS * frame_size) across all cameras = total KB/s
+        total_throughput_kbps = 0.0
+        for stream_key, stats in per_camera_stats.items():
+            fps_avg = stats.get("fps", {}).get("avg", 0)
+            frame_size_bytes = stats.get("frame_size_bytes", {}).get("avg", 0)
+            if fps_avg > 0 and frame_size_bytes > 0:
+                total_throughput_kbps += (fps_avg * frame_size_bytes) / 1024
+
+        total_throughput_mbps = total_throughput_kbps / 1024
+
+        # Log with BOLD formatting
+        B = self.BOLD
+        R = self.RESET
+
+        # Mode indicator
+        mode_str = f"SHM ({shm_format})" if use_shm else "JPEG"
+        encode_label = "Convert" if use_shm else "Encode"
+
+        logging.info(
+            f"\n{B}{'='*80}{R}\n"
+            f"{B}[STREAMING GATEWAY AGGREGATE METRICS - 1 MIN SUMMARY]{R}\n"
+            f"{B}{'='*80}{R}\n"
+            f"{B}Mode:{R} {mode_str} | "
+            f"{B}Workers:{R} {running_workers}/{num_workers} active | "
+            f"{B}Cameras:{R} {total_cameras} streaming | "
+            f"{B}Runtime:{R} {runtime:.0f}s\n"
+            f"{B}{'─'*80}{R}\n"
+            f"{B}FPS:{R} avg={avg_fps:.1f} | min={min_fps:.1f} | max={max_fps:.1f} | "
+            f"{B}TOTAL={total_fps:.1f} fps{R}\n"
+            f"{B}{'─'*80}{R}\n"
+            f"{B}LATENCY BREAKDOWN:{R}\n"
+            f"  • Read:     {avg_read_ms:.2f} ms (avg)\n"
+            f"  • {encode_label}:   {avg_encoding_ms:.2f} ms (avg)\n"
+            f"  • Write:    {avg_write_ms:.2f} ms (avg)\n"
+            f"  • {B}TOTAL:    {total_latency_ms:.2f} ms{R}\n"
+            f"{B}{'─'*80}{R}\n"
+            f"{B}THROUGHPUT:{R}\n"
+            f"  • Avg Frame Size: {avg_frame_size_kb:.1f} KB\n"
+            f"  • {B}TOTAL: {total_throughput_mbps:.2f} MB/s ({total_throughput_kbps:.1f} KB/s){R}\n"
+            f"{B}{'='*80}{R}"
+        )
+
+    def _log_aggregate_camera_streamer_bold(self, gateway_stats: Dict[str, Any]):
+        """Log aggregate metrics for CameraStreamer with BOLD formatting."""
+        camera_streamer = self.streaming_gateway.camera_streamer
+        if not camera_streamer:
+            return
+
+        stream_keys = gateway_stats.get("my_stream_keys", [])
+        transmission_stats = gateway_stats.get("transmission_stats", {})
+        runtime = gateway_stats.get("runtime_seconds", 0)
+
+        # Aggregate FPS stats
+        fps_values = []
+        read_times = []
+        write_times = []
+        encoding_times = []
+        frame_sizes = []
+
+        # Calculate total throughput properly per-camera
+        total_throughput_kbps = 0.0
+
+        for stream_key in stream_keys:
+            timing_stats = camera_streamer.statistics.get_timing_statistics(stream_key)
+            if timing_stats:
+                fps_avg = timing_stats.get("fps", {}).get("avg", 0)
+                if fps_avg > 0:
+                    fps_values.append(fps_avg)
+
+                read_ms = timing_stats.get("read_time_ms", {}).get("avg", 0)
+                write_ms = timing_stats.get("write_time_ms", {}).get("avg", 0)
+                encoding_ms = timing_stats.get("encoding_time_ms", {}).get("avg", 0)
+                frame_size_bytes = timing_stats.get("frame_size_bytes", {}).get("avg", 0)
+
+                if read_ms > 0:
+                    read_times.append(read_ms)
+                if write_ms > 0:
+                    write_times.append(write_ms)
+                if encoding_ms > 0:
+                    encoding_times.append(encoding_ms)
+                if frame_size_bytes > 0:
+                    frame_sizes.append(frame_size_bytes)
+
+                # Calculate throughput per camera (FPS * frame_size)
+                if fps_avg > 0 and frame_size_bytes > 0:
+                    total_throughput_kbps += (fps_avg * frame_size_bytes) / 1024
+
+        total_fps = sum(fps_values)
+        avg_fps = sum(fps_values) / len(fps_values) if fps_values else 0
+        min_fps = min(fps_values) if fps_values else 0
+        max_fps = max(fps_values) if fps_values else 0
+
+        avg_read_ms = sum(read_times) / len(read_times) if read_times else 0
+        avg_write_ms = sum(write_times) / len(write_times) if write_times else 0
+        avg_encoding_ms = sum(encoding_times) / len(encoding_times) if encoding_times else 0
+        total_latency_ms = avg_read_ms + avg_encoding_ms + avg_write_ms
+
+        avg_frame_size_kb = (sum(frame_sizes) / len(frame_sizes) / 1024) if frame_sizes else 0
+
+        total_throughput_mbps = total_throughput_kbps / 1024
+
+        frames_sent = transmission_stats.get("frames_sent_full", 0)
+        frames_skipped = transmission_stats.get("frames_skipped", 0)
+
+        # Log with BOLD formatting
+        B = self.BOLD
+        R = self.RESET
+
+        logging.info(
+            f"\n{B}{'='*80}{R}\n"
+            f"{B}[STREAMING GATEWAY AGGREGATE METRICS - 1 MIN SUMMARY]{R}\n"
+            f"{B}{'='*80}{R}\n"
+            f"{B}Cameras:{R} {len(stream_keys)} streaming | "
+            f"{B}Runtime:{R} {runtime:.0f}s | "
+            f"{B}Frames:{R} sent={frames_sent}, skipped={frames_skipped}\n"
+            f"{B}{'─'*80}{R}\n"
+            f"{B}FPS:{R} avg={avg_fps:.1f} | min={min_fps:.1f} | max={max_fps:.1f} | "
+            f"{B}TOTAL={total_fps:.1f} fps{R}\n"
+            f"{B}{'─'*80}{R}\n"
+            f"{B}LATENCY BREAKDOWN:{R}\n"
+            f"  • Read:     {avg_read_ms:.2f} ms (avg)\n"
+            f"  • Encode:   {avg_encoding_ms:.2f} ms (avg)\n"
+            f"  • Write:    {avg_write_ms:.2f} ms (avg)\n"
+            f"  • {B}TOTAL:    {total_latency_ms:.2f} ms{R}\n"
+            f"{B}{'─'*80}{R}\n"
+            f"{B}THROUGHPUT:{R}\n"
+            f"  • Avg Frame Size: {avg_frame_size_kb:.1f} KB\n"
+            f"  • {B}TOTAL: {total_throughput_mbps:.2f} MB/s ({total_throughput_kbps:.1f} KB/s){R}\n"
+            f"{B}{'='*80}{R}"
+        )
+
     def _log_camera_streamer_metrics(self, gateway_stats: Dict[str, Any], snapshot: Optional[Dict[str, Any]]):
         """Log metrics summary for CameraStreamer flow."""
         camera_streamer = self.streaming_gateway.camera_streamer
