@@ -62,6 +62,7 @@ class StreamingAction:
         max_restart_attempts: int = 3,
         action_id_check_interval: float = 600.0,  # 10 minutes (was 60 seconds)
         enable_event_listening: bool = True,
+        allow_empty_start: bool = True,  # Allow starting with zero cameras
     ):
         """Initialize StreamingAction.
 
@@ -74,6 +75,7 @@ class StreamingAction:
             max_restart_attempts: Maximum number of restart attempts before giving up
             action_id_check_interval: Interval in seconds between checks to verify action ID matches streaming gateway
             enable_event_listening: Enable dynamic event listening for configuration updates
+            allow_empty_start: Allow starting with zero cameras (default True). Cameras can be added dynamically.
         """
         if not session:
             raise ValueError("Session is required")
@@ -142,6 +144,7 @@ class StreamingAction:
         self.max_restart_attempts = max_restart_attempts
         self.action_id_check_interval = action_id_check_interval
         self.enable_event_listening = enable_event_listening
+        self.allow_empty_start = allow_empty_start
 
         # Initialize utility and gateway objects
         self.gateway_util = StreamingGatewayUtil(
@@ -246,11 +249,18 @@ class StreamingAction:
             input_streams = self.gateway_util.get_input_streams()
 
             if not input_streams:
-                raise RuntimeError(
-                    "No input streams configured for this streaming gateway"
-                )
-
-            logging.info("Found %d input streams configured", len(input_streams))
+                if self.allow_empty_start:
+                    logging.warning(
+                        "No input streams configured - starting with zero cameras. "
+                        "Cameras can be added dynamically via event listener."
+                    )
+                    input_streams = []
+                else:
+                    raise RuntimeError(
+                        "No input streams configured for this streaming gateway"
+                    )
+            else:
+                logging.info("Found %d input streams configured", len(input_streams))
 
             # Create StreamingGateway with fetched configuration
             logging.info("Creating StreamingGateway instance...")
@@ -264,6 +274,7 @@ class StreamingAction:
                 force_restart=True,  # Always force restart for orchestrator
                 enable_event_listening=self.enable_event_listening,  # Enable dynamic event updates
                 action_id=self.action_id,  # Pass action_id for API requests
+                allow_empty_start=self.allow_empty_start,  # Allow starting with zero cameras
             )
 
             # Start streaming
