@@ -1,5 +1,5 @@
 import logging
-import base64
+import os
 import time
 import threading
 import atexit
@@ -15,6 +15,9 @@ from .streaming_gateway_utils import (
 from .event_listener import EventListener
 from .dynamic_camera_manager import DynamicCameraManager, DynamicCameraManagerForWorkers
 
+USE_FFMPEG = os.getenv("USE_FFMPEG", "false").lower() == "true"
+USE_GSTREAMER = os.getenv("USE_GSTREAMER", "false").lower() == "true"
+
 # GStreamer imports (optional - graceful degradation)
 GSTREAMER_AVAILABLE = False
 try:
@@ -30,6 +33,17 @@ except (ImportError, ValueError):
     # ValueError: gi.require_version fails when GStreamer not installed
     pass
 
+# FFmpeg imports (optional - graceful degradation)
+FFMPEG_AVAILABLE = False
+try:
+    from .camera_streamer.ffmpeg_config import FFmpegConfig, is_ffmpeg_available
+    from .camera_streamer.ffmpeg_camera_streamer import FFmpegCameraStreamer
+    from .camera_streamer.ffmpeg_worker_manager import FFmpegWorkerManager
+    FFMPEG_AVAILABLE = is_ffmpeg_available()
+except (ImportError, FileNotFoundError):
+    # FFmpeg not available or not installed
+    pass
+
 
 class StreamingGateway:
     """Simplified streaming gateway for managing camera streams."""
@@ -59,6 +73,22 @@ class StreamingGateway:
         gstreamer_codec: str = "h264",  # h264, h265
         gstreamer_preset: str = "low-latency",  # NVENC preset
         gstreamer_gpu_id: int = 0,  # GPU device ID for NVENC
+        # Platform-specific GStreamer options
+        gstreamer_platform: str = "auto",  # auto, jetson, desktop-gpu, intel, amd, cpu
+        gstreamer_use_hardware_decode: bool = True,  # Use hardware decode (nvv4l2decoder, nvdec, vaapi)
+        gstreamer_use_hardware_jpeg: bool = True,  # Use hardware JPEG (nvjpegenc, vaapijpegenc)
+        gstreamer_jetson_use_nvmm: bool = True,  # Use NVMM zero-copy on Jetson
+        gstreamer_frame_optimizer_mode: str = "hash-only",  # hash-only, dual-appsink, disabled
+        gstreamer_fallback_on_error: bool = True,  # Gracefully fallback to CPU pipeline on error
+        gstreamer_verbose_logging: bool = False,  # Verbose pipeline logging for debugging
+        # FFmpeg options
+        use_ffmpeg: bool = USE_FFMPEG,          # Use FFmpeg subprocess-based encoding
+        ffmpeg_hwaccel: str = "auto",      # Hardware acceleration: auto, cuda, vaapi, none
+        ffmpeg_threads: int = 1,           # FFmpeg decode threads per stream
+        ffmpeg_low_latency: bool = True,   # Enable low-latency flags
+        ffmpeg_pixel_format: str = "bgr24",# Output pixel format
+        # SHM configuration (centralized)
+        shm_slot_count: int = 1000,        # Ring buffer size per camera (increased for consumer lag)
     ):
         """Initialize StreamingGateway.
 
@@ -81,6 +111,19 @@ class StreamingGateway:
             gstreamer_codec: GStreamer codec (h264, h265)
             gstreamer_preset: NVENC preset for hardware encoding
             gstreamer_gpu_id: GPU device ID for NVENC hardware encoding
+            gstreamer_platform: Platform override (auto, jetson, desktop-gpu, intel, amd, cpu)
+            gstreamer_use_hardware_decode: Enable hardware decode (nvv4l2decoder, nvdec, vaapi)
+            gstreamer_use_hardware_jpeg: Enable hardware JPEG encoding when available
+            gstreamer_jetson_use_nvmm: Use NVMM zero-copy memory on Jetson devices
+            gstreamer_frame_optimizer_mode: Frame optimization mode (hash-only, dual-appsink, disabled)
+            gstreamer_fallback_on_error: Automatically fallback to CPU pipeline on hardware errors
+            gstreamer_verbose_logging: Enable verbose pipeline construction logging
+            use_ffmpeg: Use FFmpeg subprocess-based encoding (alternative to OpenCV/GStreamer)
+            ffmpeg_hwaccel: FFmpeg hardware acceleration (auto, cuda, vaapi, none)
+            ffmpeg_threads: Number of FFmpeg decode threads per stream
+            ffmpeg_low_latency: Enable FFmpeg low-latency flags
+            ffmpeg_pixel_format: Output pixel format (bgr24, rgb24, nv12)
+            shm_slot_count: Number of frame slots per camera ring buffer for SHM mode (default: 300)
         """
         if not session:
             raise ValueError("Session is required")
@@ -102,6 +145,14 @@ class StreamingGateway:
         self.gstreamer_codec = gstreamer_codec
         self.gstreamer_preset = gstreamer_preset
         self.gstreamer_gpu_id = gstreamer_gpu_id
+        # Platform-specific GStreamer configuration
+        self.gstreamer_platform = gstreamer_platform
+        self.gstreamer_use_hardware_decode = gstreamer_use_hardware_decode
+        self.gstreamer_use_hardware_jpeg = gstreamer_use_hardware_jpeg
+        self.gstreamer_jetson_use_nvmm = gstreamer_jetson_use_nvmm
+        self.gstreamer_frame_optimizer_mode = gstreamer_frame_optimizer_mode
+        self.gstreamer_fallback_on_error = gstreamer_fallback_on_error
+        self.gstreamer_verbose_logging = gstreamer_verbose_logging
         
         # Validate GStreamer availability if requested
         if use_gstreamer and not GSTREAMER_AVAILABLE:
@@ -110,6 +161,27 @@ class StreamingGateway:
                 "Install with: pip install PyGObject && apt-get install gstreamer1.0-plugins-*"
             )
 
+        # FFmpeg configuration
+        self.use_ffmpeg = use_ffmpeg
+        self.ffmpeg_hwaccel = ffmpeg_hwaccel
+        self.ffmpeg_threads = ffmpeg_threads
+        self.ffmpeg_low_latency = ffmpeg_low_latency
+        self.ffmpeg_pixel_format = ffmpeg_pixel_format
+
+        # SHM configuration (centralized for all workers)
+        self.shm_slot_count = shm_slot_count
+
+        # Validate FFmpeg availability if requested
+        if use_ffmpeg and not FFMPEG_AVAILABLE:
+            raise RuntimeError(
+                "FFmpeg requested but not available. "
+                "Install FFmpeg from https://ffmpeg.org/download.html"
+            )
+
+        # Validate exclusive backend selection
+        if use_gstreamer and use_ffmpeg:
+            raise ValueError("Cannot enable both GStreamer and FFmpeg backends")
+
         # Initialize utility for API interactions
         self.gateway_util = StreamingGatewayUtil(session, streaming_gateway_id, server_id, action_id=action_id)
 
@@ -156,13 +228,84 @@ class StreamingGateway:
                 raise ValueError(f"Input config {i} must be an InputStream instance")
 
         # Initialize streaming backend based on configuration
-        # Options: use_gstreamer (GStreamer-based), use_async_workers (AsyncCameraWorker), or CameraStreamer
+        # Options: use_ffmpeg, use_gstreamer, use_async_workers (AsyncCameraWorker), or CameraStreamer
         self.camera_streamer: Optional[CameraStreamer] = None
         self.worker_manager: Optional[WorkerManager] = None
         self.gstreamer_streamer: Optional[Any] = None  # GStreamerCameraStreamer
         self.gstreamer_worker_manager: Optional[Any] = None  # GStreamerWorkerManager
+        self.ffmpeg_streamer: Optional[Any] = None  # FFmpegCameraStreamer
+        self.ffmpeg_worker_manager: Optional[Any] = None  # FFmpegWorkerManager
 
-        if self.use_gstreamer:
+        if self.use_ffmpeg:
+            # FFmpeg-based streaming flow
+            # Build stream config for workers
+            stream_config = build_stream_config(
+                gateway_util=self.gateway_util,
+                server_type=server_type,
+                service_id=streaming_gateway_id,
+                stream_maxlen=self.shm_slot_count,
+            )
+
+            # Create FFmpeg configuration
+            ffmpeg_config = FFmpegConfig(
+                hwaccel=ffmpeg_hwaccel,
+                threads=ffmpeg_threads,
+                low_latency=ffmpeg_low_latency,
+                pixel_format=ffmpeg_pixel_format,
+            )
+
+            if self.use_async_workers:
+                # FFmpeg with worker processes
+                logging.info(
+                    f"Initializing FFmpeg worker flow - hwaccel: {ffmpeg_hwaccel}, "
+                    f"threads: {ffmpeg_threads}"
+                )
+
+                # Convert InputStream configs to camera_config dicts
+                camera_configs = [
+                    input_stream_to_camera_config(inp) for inp in self.inputs_config
+                ]
+
+                self.ffmpeg_worker_manager = FFmpegWorkerManager(
+                    camera_configs=camera_configs,
+                    stream_config=stream_config,
+                    num_workers=num_workers,
+                    max_cameras_per_worker=max_cameras_per_worker,
+                    ffmpeg_config=ffmpeg_config,
+                    shm_slot_count=self.shm_slot_count,
+                )
+
+                # Initialize dynamic camera manager for FFmpeg workers
+                self.camera_manager = DynamicCameraManagerForWorkers(
+                    worker_manager=self.ffmpeg_worker_manager,
+                    streaming_gateway_id=streaming_gateway_id,
+                    session=self.session,
+                    streaming_gateway=self,
+                )
+            else:
+                # FFmpeg single-threaded mode
+                logging.info(
+                    f"Initializing FFmpeg CameraStreamer - hwaccel: {ffmpeg_hwaccel}"
+                )
+
+                self.ffmpeg_streamer = FFmpegCameraStreamer(
+                    session=self.session,
+                    service_id=streaming_gateway_id,
+                    server_type=server_type,
+                    video_codec=video_codec,
+                    gateway_util=self.gateway_util,
+                    ffmpeg_config=ffmpeg_config,
+                )
+
+                # Initialize dynamic camera manager for FFmpeg streamer
+                self.camera_manager = DynamicCameraManager(
+                    camera_streamer=self.ffmpeg_streamer,
+                    streaming_gateway_id=streaming_gateway_id,
+                    session=self.session,
+                    streaming_gateway=self,
+                )
+
+        elif self.use_gstreamer:
             # GStreamer-based encoding flow
             if self.use_async_workers:
                 # GStreamer with worker processes
@@ -176,6 +319,7 @@ class StreamingGateway:
                     gateway_util=self.gateway_util,
                     server_type=server_type,
                     service_id=streaming_gateway_id,
+                    stream_maxlen=self.shm_slot_count,
                 )
 
                 # Convert InputStream configs to camera_config dicts
@@ -192,6 +336,13 @@ class StreamingGateway:
                     gstreamer_codec=gstreamer_codec,
                     gstreamer_preset=gstreamer_preset,
                     gpu_id=gstreamer_gpu_id,
+                    platform=gstreamer_platform,
+                    use_hardware_decode=gstreamer_use_hardware_decode,
+                    use_hardware_jpeg=gstreamer_use_hardware_jpeg,
+                    jetson_use_nvmm=gstreamer_jetson_use_nvmm,
+                    frame_optimizer_mode=gstreamer_frame_optimizer_mode,
+                    fallback_on_error=gstreamer_fallback_on_error,
+                    verbose_pipeline_logging=gstreamer_verbose_logging,
                 )
 
                 # Initialize dynamic camera manager for GStreamer workers
@@ -214,6 +365,13 @@ class StreamingGateway:
                     codec=gstreamer_codec,
                     preset=gstreamer_preset,
                     gpu_id=gstreamer_gpu_id,
+                    platform=gstreamer_platform,
+                    use_hardware_decode=gstreamer_use_hardware_decode,
+                    use_hardware_jpeg=gstreamer_use_hardware_jpeg,
+                    jetson_use_nvmm=gstreamer_jetson_use_nvmm,
+                    frame_optimizer_mode=gstreamer_frame_optimizer_mode,
+                    fallback_on_error=gstreamer_fallback_on_error,
+                    verbose_pipeline_logging=gstreamer_verbose_logging,
                 )
 
                 self.gstreamer_streamer = GStreamerCameraStreamer(
@@ -243,6 +401,7 @@ class StreamingGateway:
                 gateway_util=self.gateway_util,
                 server_type=server_type,
                 service_id=streaming_gateway_id,
+                stream_maxlen=self.shm_slot_count,
             )
 
             # Convert InputStream configs to camera_config dicts
@@ -255,6 +414,7 @@ class StreamingGateway:
                 stream_config=stream_config,
                 num_workers=num_workers,
                 max_cameras_per_worker=max_cameras_per_worker,
+                shm_slot_count=self.shm_slot_count,
             )
 
             # Initialize dynamic camera manager for workers
@@ -377,7 +537,12 @@ class StreamingGateway:
         self._register_as_active()
 
         try:
-            if self.use_gstreamer:
+            if self.use_ffmpeg:
+                if self.use_async_workers:
+                    success = self._start_ffmpeg_worker_streaming()
+                else:
+                    success = self._start_ffmpeg_streamer_streaming()
+            elif self.use_gstreamer:
                 if self.use_async_workers:
                     success = self._start_gstreamer_worker_streaming()
                 else:
@@ -589,6 +754,98 @@ class StreamingGateway:
 
         return True
 
+    def _start_ffmpeg_worker_streaming(self) -> bool:
+        """Start streaming using FFmpeg worker processes.
+
+        Returns:
+            bool: True if started successfully, False otherwise
+        """
+        num_cameras = len(self.inputs_config) if self.inputs_config else 0
+        logging.info(
+            f"Starting FFmpeg worker streaming with {num_cameras} cameras "
+            f"(hwaccel: {self.ffmpeg_hwaccel}, threads: {self.ffmpeg_threads})"
+        )
+
+        # Build stream key mappings
+        if self.inputs_config:
+            for i, input_config in enumerate(self.inputs_config):
+                stream_key = input_config.camera_key or f"stream_{i}"
+                camera_id = input_config.camera_id or stream_key
+                self._stream_key_to_camera_id[stream_key] = camera_id
+                self._my_stream_keys.add(stream_key)
+
+        # Start the FFmpeg worker manager
+        try:
+            self.ffmpeg_worker_manager.start()
+            logging.info(
+                f"Started FFmpegWorkerManager with {self.num_workers} workers, "
+                f"{num_cameras} cameras"
+            )
+            return True
+        except Exception as exc:
+            logging.error(f"Failed to start FFmpegWorkerManager: {exc}", exc_info=True)
+            return False
+
+    def _start_ffmpeg_streamer_streaming(self) -> bool:
+        """Start streaming using FFmpeg CameraStreamer (single-threaded).
+
+        Returns:
+            bool: True if started successfully, False otherwise
+        """
+        num_cameras = len(self.inputs_config) if self.inputs_config else 0
+        logging.info(
+            f"Starting FFmpeg CameraStreamer with {num_cameras} cameras "
+            f"(hwaccel: {self.ffmpeg_hwaccel})"
+        )
+
+        # If no cameras, return success (ready for dynamic cameras)
+        if not self.inputs_config:
+            logging.info("No cameras to start - awaiting dynamic camera addition")
+            return True
+
+        started_streams = []
+
+        for i, input_config in enumerate(self.inputs_config):
+            stream_key = input_config.camera_key or f"stream_{i}"
+
+            # Store camera_id mapping
+            camera_id = input_config.camera_id or stream_key
+            self._stream_key_to_camera_id[stream_key] = camera_id
+
+            # Register topic
+            topic = input_config.camera_input_topic
+            if not topic:
+                topic = f"{camera_id}_input_topic"
+                logging.warning(f"No input topic for camera {input_config.camera_key}, using default: {topic}")
+
+            self.ffmpeg_streamer.register_stream_topic(stream_key, topic)
+
+            # Start streaming
+            success = self.ffmpeg_streamer.start_background_stream(
+                input=input_config.source,
+                fps=input_config.fps,
+                stream_key=stream_key,
+                stream_group_key=input_config.camera_group_key,
+                quality=input_config.quality,
+                width=input_config.width,
+                height=input_config.height,
+                simulate_video_file_stream=input_config.simulate_video_file_stream,
+                camera_location=input_config.camera_location,
+            )
+
+            if not success:
+                logging.error(f"Failed to start FFmpeg streaming for {input_config.source}")
+                if started_streams:
+                    logging.info("Stopping already started streams")
+                    self.stop_streaming()
+                return False
+
+            started_streams.append(stream_key)
+            self._my_stream_keys.add(stream_key)
+            logging.info(f"Started FFmpeg streaming for camera: {input_config.camera_key}")
+
+        return True
+
     def stop_streaming(self):
         """Stop all streaming operations."""
         with self._state_lock:
@@ -610,7 +867,38 @@ class StreamingGateway:
                 logging.error(f"Error stopping event listener: {exc}")
 
         # Stop streaming backend based on which flow is active
-        if self.use_gstreamer:
+        if self.use_ffmpeg:
+            # Stop FFmpeg backends
+            if self.use_async_workers:
+                # Stop FFmpegWorkerManager
+                if self.ffmpeg_worker_manager:
+                    try:
+                        logging.info("Stopping FFmpegWorkerManager")
+                        self.ffmpeg_worker_manager.stop()
+                        logging.info("FFmpeg worker manager stopped")
+                    except Exception as exc:
+                        logging.error(f"Error stopping FFmpegWorkerManager: {exc}")
+            else:
+                # Stop FFmpegCameraStreamer
+                if self.ffmpeg_streamer:
+                    try:
+                        logging.info("Stopping FFmpegCameraStreamer")
+                        self.ffmpeg_streamer.stop_streaming()
+                        # Reset statistics for clean restart
+                        self.ffmpeg_streamer.reset_transmission_stats()
+                        # Clear pipeline references
+                        if hasattr(self.ffmpeg_streamer, 'pipelines'):
+                            self.ffmpeg_streamer.pipelines.clear()
+                        # Join streaming threads with timeout
+                        if hasattr(self.ffmpeg_streamer, 'streaming_threads'):
+                            for thread in self.ffmpeg_streamer.streaming_threads:
+                                if thread.is_alive():
+                                    thread.join(timeout=5.0)
+                            self.ffmpeg_streamer.streaming_threads.clear()
+                        logging.info("FFmpeg cleanup completed")
+                    except Exception as exc:
+                        logging.error(f"Error stopping FFmpeg streaming: {exc}")
+        elif self.use_gstreamer:
             # Stop GStreamer backends
             if self.use_async_workers:
                 # Stop GStreamerWorkerManager
@@ -711,15 +999,46 @@ class StreamingGateway:
         stats["event_listening_enabled"] = self.enable_event_listening
         stats["use_async_workers"] = self.use_async_workers
         stats["use_gstreamer"] = self.use_gstreamer
+        stats["use_ffmpeg"] = self.use_ffmpeg
 
         # Add backend-specific statistics
-        if self.use_gstreamer:
+        if self.use_ffmpeg:
+            # FFmpeg statistics
+            stats["ffmpeg_config"] = {
+                "hwaccel": self.ffmpeg_hwaccel,
+                "threads": self.ffmpeg_threads,
+                "low_latency": self.ffmpeg_low_latency,
+                "pixel_format": self.ffmpeg_pixel_format,
+            }
+            if self.use_async_workers:
+                # FFmpegWorkerManager statistics
+                if self.ffmpeg_worker_manager:
+                    try:
+                        stats["worker_stats"] = self.ffmpeg_worker_manager.get_worker_statistics()
+                        stats["camera_assignments"] = self.ffmpeg_worker_manager.get_camera_assignments()
+                    except Exception as exc:
+                        logging.warning(f"Failed to get FFmpeg worker stats: {exc}")
+            else:
+                # FFmpegCameraStreamer statistics
+                if self.ffmpeg_streamer:
+                    try:
+                        stats["transmission_stats"] = self.ffmpeg_streamer.get_transmission_stats()
+                    except Exception as exc:
+                        logging.warning(f"Failed to get FFmpeg transmission stats: {exc}")
+        elif self.use_gstreamer:
             # GStreamer statistics
             stats["gstreamer_config"] = {
                 "encoder": self.gstreamer_encoder,
                 "codec": self.gstreamer_codec,
                 "preset": self.gstreamer_preset,
                 "gpu_id": self.gstreamer_gpu_id,
+                "platform": self.gstreamer_platform,
+                "use_hardware_decode": self.gstreamer_use_hardware_decode,
+                "use_hardware_jpeg": self.gstreamer_use_hardware_jpeg,
+                "jetson_use_nvmm": self.gstreamer_jetson_use_nvmm,
+                "frame_optimizer_mode": self.gstreamer_frame_optimizer_mode,
+                "fallback_on_error": self.gstreamer_fallback_on_error,
+                "verbose_logging": self.gstreamer_verbose_logging,
             }
             if self.use_async_workers:
                 # GStreamerWorkerManager statistics
@@ -792,11 +1111,25 @@ class StreamingGateway:
             "use_async_workers": self.use_async_workers,
             "num_workers": self.num_workers,
             "max_cameras_per_worker": self.max_cameras_per_worker,
+            # FFmpeg configuration
+            "use_ffmpeg": self.use_ffmpeg,
+            "ffmpeg_hwaccel": self.ffmpeg_hwaccel,
+            "ffmpeg_threads": self.ffmpeg_threads,
+            "ffmpeg_low_latency": self.ffmpeg_low_latency,
+            "ffmpeg_pixel_format": self.ffmpeg_pixel_format,
+            # GStreamer configuration
             "use_gstreamer": self.use_gstreamer,
             "gstreamer_encoder": self.gstreamer_encoder,
             "gstreamer_codec": self.gstreamer_codec,
             "gstreamer_preset": self.gstreamer_preset,
             "gstreamer_gpu_id": self.gstreamer_gpu_id,
+            "gstreamer_platform": self.gstreamer_platform,
+            "gstreamer_use_hardware_decode": self.gstreamer_use_hardware_decode,
+            "gstreamer_use_hardware_jpeg": self.gstreamer_use_hardware_jpeg,
+            "gstreamer_jetson_use_nvmm": self.gstreamer_jetson_use_nvmm,
+            "gstreamer_frame_optimizer_mode": self.gstreamer_frame_optimizer_mode,
+            "gstreamer_fallback_on_error": self.gstreamer_fallback_on_error,
+            "gstreamer_verbose_logging": self.gstreamer_verbose_logging,
         }
 
     def _emergency_cleanup(self):
