@@ -190,29 +190,70 @@ class StreamingGatewayUtil:
         # Identify FILE cameras that need simulated stream URLs
         file_cameras = [c for c in cameras if c.get("protocolType") == "FILE"]
 
-        # Fetch simulated URLs concurrently
+        # Fetch simulated URLs concurrently, with caching for cameras sharing the same simulationVideoPath
         stream_url_lookup = {}
         if file_cameras:
-            max_workers = min(len(file_cameras), 100)  # Max 100 concurrent requests
+            # Group cameras by simulationVideoPath to avoid redundant API calls
+            # Only group non-empty paths; empty paths need individual API calls
+            path_to_cameras: Dict[str, List[dict]] = {}
+            cameras_needing_api_call = []
+            
+            for camera in file_cameras:
+                sim_path = camera.get("simulationVideoPath", "")
+                if sim_path:  # Non-empty path - can potentially share URL
+                    if sim_path not in path_to_cameras:
+                        path_to_cameras[sim_path] = []
+                        # First camera with this path will make the API call
+                        cameras_needing_api_call.append(camera)
+                    path_to_cameras[sim_path].append(camera)
+                else:
+                    # Empty path - needs its own API call
+                    cameras_needing_api_call.append(camera)
+            
+            logging.info(
+                f"FILE cameras: {len(file_cameras)} total, {len(cameras_needing_api_call)} unique API calls needed "
+                f"({len(file_cameras) - len(cameras_needing_api_call)} will use cached URLs)"
+            )
+            
+            max_workers = min(len(cameras_needing_api_call), 100)  # Max 100 concurrent requests
             start = time.time()
+            
+            # Cache to store presigned URL by simulationVideoPath
+            path_url_cache: Dict[str, str] = {}
+            
             with ThreadPoolExecutor(max_workers=max_workers) as executor:
                 future_to_camera = {
-                    executor.submit(self.get_simulated_stream_url, c["id"]): c["id"]
-                    for c in file_cameras
+                    executor.submit(self.get_simulated_stream_url, c["id"]): c
+                    for c in cameras_needing_api_call
                 }
                 for future in as_completed(future_to_camera):
-                    camera_id = future_to_camera[future]
+                    camera = future_to_camera[future]
+                    camera_id = camera["id"]
+                    sim_path = camera.get("simulationVideoPath", "")
                     try:
                         result = future.result()
                         if result and result.get("url"):
-                            stream_url_lookup[camera_id] = result["url"]
+                            presigned_url = result["url"]
+                            stream_url_lookup[camera_id] = presigned_url
+                            # Cache the URL for this simulationVideoPath
+                            if sim_path:
+                                path_url_cache[sim_path] = presigned_url
                     except Exception as e:
                         logging.warning(f"Failed to get stream URL for camera {camera_id}: {e}")
+            
+            # Apply cached URLs to cameras that share the same simulationVideoPath
+            for sim_path, cameras_with_path in path_to_cameras.items():
+                if sim_path in path_url_cache:
+                    cached_url = path_url_cache[sim_path]
+                    for camera in cameras_with_path:
+                        # Only set if not already set (the first camera already has it)
+                        if camera["id"] not in stream_url_lookup:
+                            stream_url_lookup[camera["id"]] = cached_url
 
             urls_time = time.time() - start
             logging.info(
-                f"Concurrent fetch of {len(file_cameras)} stream URLs took {urls_time:.3f}s "
-                f"(max_workers={max_workers})"
+                f"Concurrent fetch of {len(cameras_needing_api_call)} stream URLs took {urls_time:.3f}s "
+                f"(max_workers={max_workers}, cached {len(file_cameras) - len(cameras_needing_api_call)} URLs)"
             )
 
         input_streams = []
@@ -392,12 +433,12 @@ class StreamingGatewayUtil:
                     data = response.get("data")
                     if (
                         data
-                        and data.get("host")
+                        # TODO: Check why BE is giving host as empty while in the DB it is localhost
                         and data.get("port")
                         and data.get("status") == "running"
                     ):
                         return {
-                            "host": data["host"],
+                            "host": data.get("host") or "localhost",
                             "port": int(data["port"]),
                             "password": data.get("password", ""),
                             "username": data.get("username"),
@@ -540,7 +581,8 @@ def input_stream_to_camera_config(input_stream: InputStream) -> Dict:
 def build_stream_config(
     gateway_util: StreamingGatewayUtil,
     server_type: str,
-    service_id: str
+    service_id: str,
+    stream_maxlen: int = None
 ) -> Dict:
     """Build stream_config dict for WorkerManager from gateway connection info.
 
@@ -551,6 +593,7 @@ def build_stream_config(
         gateway_util: StreamingGatewayUtil instance for API calls
         server_type: 'kafka' or 'redis'
         service_id: Streaming gateway ID (used as service_id)
+        stream_maxlen: Maximum entries per Redis stream (approximate mode, default: None = unlimited)
 
     Returns:
         Dict with connection configuration for WorkerManager
@@ -574,6 +617,7 @@ def build_stream_config(
             'enable_batching': True,
             'batch_size': 10,
             'batch_timeout': 0.01,
+            'stream_maxlen': stream_maxlen,  # Redis XADD maxlen (approximate mode)
         })
 
     return stream_config
\ No newline at end of file
